<?php
/**
 * @package	RSFirewall!
 * @copyright	(c) 2009 - 2022 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Uri\Uri;

/**
 * RSFirewall! RSFinder! Plugin.
 */
class plgRsfinderRsfirewall extends CMSPlugin
{
    const EXTENSION = 'com_rsfirewall';
	
	 /**
     * @var bool
     */
    protected $autoloadLanguage = true;

    protected function canRun() {
        // Check if the component exists and is enabled - in backend
        return (
            file_exists(JPATH_ADMINISTRATOR . '/components/'.self::EXTENSION.'/rsfirewall.php') &&
            Factory::getApplication()->isClient('administrator') &&
            ComponentHelper::getComponent(self::EXTENSION, true)->enabled
        );
    }

    public function onRsfinderGetResults(&$tmp_results, $search, $limit, &$index)
    {
        if (!$this->canRun())
        {
            return;
        }

        // verify the index first, no need to continue if the limit is reached
        if ($index >= $limit)
        {
            return;
        }

        // Get the current User
        $user = Factory::getApplication()->getIdentity();
        $access  = $user->authorise('core.manage', self::EXTENSION) && $user->authorise('core.edit', self::EXTENSION);

        // if there is no access, skip the results
        if (!$access)
        {
            return;
        }

        $select_in = $this->params->get('searchList');

        // workaround for stored version
        if (is_string($select_in))
        {
            $select_in = explode(',', $select_in);
        }

        $output_results = array('dynamic' => array(), 'fixed' => array());

        // Get DBO.
        $db	= Factory::getContainer()->get('DatabaseDriver');

        if (in_array('lists', $select_in))
        {

            $query = $db->getQuery(true);
            $query->select(
                [
                    $db->qn('rsfl.id', 'value'),
                    $db->qn('rsfl.ip', 'text'),
                    $db->qn('rsfl.type', 'type')
                ]
            )
                ->from($db->qn('#__rsfirewall_lists', 'rsfl'))
                ->where($db->qn('rsfl.ip') . " LIKE ".$db->q('%'.$search.'%'))
                ->setLimit($limit);

            $db->setQuery($query);
            $results = $db->loadObjectList();


            if (!empty($results))
            {
                foreach($results as $result)
                {
                    // verify the index first
                    if ($index >= $limit)
                    {
                        break;
                    }

                    $output_result = array(
                        'title' => $this->escape($result->text).' '.Text::_(($result->type ? 'PLG_RSFINDER_RSFIREWALL_EDIT_SAFELIST_ENTRY' : 'PLG_RSFINDER_RSFIREWALL_EDIT_BLOCKLIST_ENTRY')),
                        'link'  => Uri::root().'administrator/index.php?option=com_rsfirewall&task=list.edit&id='.$result->value,
                        'hits'  => 0
                    );

                    $output_results['dynamic'][] = $output_result;
                    $index ++;
                }
            }

        }

        if ($index < $limit && in_array('exceptions', $select_in))
        {

            $query = $db->getQuery(true);
            $query->select(
                [
                    $db->qn('rsfe.id', 'value'),
                    $db->qn('rsfe.match', 'text'),
                ]
            )
                ->from($db->qn('#__rsfirewall_exceptions', 'rsfe'))
                ->where($db->qn('rsfe.match') . " LIKE ".$db->q('%'.$search.'%'))
                ->setLimit($limit);

            $db->setQuery($query);
            $results = $db->loadObjectList();


            if (!empty($results))
            {
                foreach($results as $result)
                {
                    // verify the index first
                    if ($index >= $limit)
                    {
                        break;
                    }

                    $output_result = array(
                        'title' => $this->escape($result->text).' '.Text::_('PLG_RSFINDER_RSFIREWALL_EDIT_EXCEPTIONS'),
                        'link'  => Uri::root().'administrator/index.php?option=com_rsfirewall&task=exception.edit&id='.$result->value,
                        'hits'  => 0
                    );

                    $output_results['dynamic'][] = $output_result;
                    $index ++;
                }
            }

        }

        // The access here is checked in the RSFinder! System plugin
        if ($index < $limit && $this->params->get('system_check', 1))
        {
            $output_results['fixed'][] = array(
                'title' => Text::_('PLG_RSFINDER_RSFIREWALL_SYSTEM_CHECK_PAGE'),
                'link'  => Uri::root().'administrator/index.php?option=com_rsfirewall&view=check',
                'hits'  => 0
            );
        }

        if ($index < $limit && $this->params->get('database_check', 1))
        {
            $output_results['fixed'][] = array(
                'title' => Text::_('PLG_RSFINDER_RSFIREWALL_DATABASE_CHECK_PAGE'),
                'link'  => Uri::root().'administrator/index.php?option=com_rsfirewall&view=dbcheck',
                'hits'  => 0
            );
        }

        if ($index < $limit && $this->params->get('system_logs', 1))
        {
            $output_results['fixed'][] = array(
                'title' => Text::_('PLG_RSFINDER_RSFIREWALL_SYSTEM_LOGS_PAGE'),
                'link'  => Uri::root().'administrator/index.php?option=com_rsfirewall&view=logs',
                'hits'  => 0
            );
        }

        if ($index < $limit && $this->params->get('lists', 1))
        {
            $output_results['fixed'][] = array(
                'title' => Text::_('PLG_RSFINDER_RSFIREWALL_LIST_PAGE'),
                'link'  => Uri::root().'administrator/index.php?option=com_rsfirewall&view=lists',
                'hits'  => 0
            );
        }
        if ($index < $limit && $this->params->get('exceptions', 1))
        {
            $output_results['fixed'][] = array(
                'title' => Text::_('PLG_RSFINDER_RSFIREWALL_EXCEPTIONS_PAGE'),
                'link'  => Uri::root().'administrator/index.php?option=com_rsfirewall&view=exceptions',
                'hits'  => 0
            );
        }

        if ($index < $limit && $this->params->get('configuration', 1))
        {
            $output_results['fixed'][] = array(
                'title' => Text::_('PLG_RSFINDER_RSFIREWALL_CONFIGURATION_PAGE'),
                'link'  => Uri::root().'administrator/index.php?option=com_rsfirewall&view=configuration',
                'hits'  => 0
            );
        }

        $tmp_results[] = $output_results;
    }

    protected function escape($string) {
        $string = (string) $string;
        return htmlentities($string, ENT_COMPAT, 'utf-8');
    }
        
}